unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, AppEvnts, DirectDraw, DDUtil;

type
  TVertol=record
   vLeft:Integer;
   vTop:Integer;
   vState:Integer;
   vWaitTime:Integer;
  end;

  TForm1 = class(TForm)
    ApplicationEvents1: TApplicationEvents;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure ApplicationEvents1Restore(Sender: TObject);
    procedure ApplicationEvents1Deactivate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
    procedure ApplicationEvents1Activate(Sender: TObject);
  private
    { Private declarations }
    FActive           : Boolean;
    FDirectDraw : IDirectDraw7;
    FPrimarySurface   : IDirectDrawSurface7;
    FBackGround       : IDirectDrawSurface7;
    FImageSurface     : IDirectDrawSurface7;
    FTransparentImage : IDirectDrawSurface7;
    FVertolImage      : IDirectDrawSurface7;
    MouseX, MouseY    : Integer;
    Vertol1           : TVertol;
    procedure ErrorOut(hRet : HRESULT);
    procedure InitVertol;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.ErrorOut(hRet: HRESULT);
begin
 MessageBox(0, PChar(DDErrorString(hRet)), '', MB_OK or MB_ICONSTOP);
end;

procedure TForm1.FormCreate(Sender: TObject);
var
 hRet : HRESULT;
 SurfaceDesc : TDDSurfaceDesc2;
 fcaps : TDDSCaps2;
begin
 FPrimarySurface := nil;
 FDirectDraw := nil;
 FBackGround :=nil;
 FImageSurface :=nil;
 FTransparentImage := nil;
 FVertolImage:=nil;
 MouseX:=0;
 MouseY:=0;

 hRet := DirectDrawCreateEx (nil, FDirectDraw, IDirectDraw7, nil);
 if hRet <> DD_OK then
  begin
   ErrorOut(hRet);
   Close;
   Exit;
  end;

 hRet := FDirectDraw.SetCooperativeLevel(Handle, DDSCL_FULLSCREEN or DDSCL_EXCLUSIVE);
 if hRet <> DD_OK then
  begin
   ErrorOut(hRet);
   Close;
   Exit;
  end;

 hRet := FDirectDraw.SetDisplayMode (800, 600, 16, 0, 0);
 if hRet <> DD_OK then
  begin
   ErrorOut(hRet);
   Close;
   Exit;
  end;

 FillChar (SurfaceDesc, SizeOf(SurfaceDesc), 0);
 SurfaceDesc.dwSize := SizeOf(SurfaceDesc);
 SurfaceDesc.dwFlags := DDSD_CAPS or DDSD_BACKBUFFERCOUNT;
 SurfaceDesc.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE or DDSCAPS_FLIP or DDSCAPS_COMPLEX;
 SurfaceDesc.dwBackBufferCount := 1;
 hRet := FDirectDraw.CreateSurface(SurfaceDesc, FPrimarySurface, nil);
 if hRet <> DD_OK then
  begin
   ErrorOut(hRet);
   Close;
   Exit;
  end;

 FImageSurface := DDLoadBitmap(FDirectDraw, '1.bmp', 0, 0);
 if FImageSurface = nil then
  begin
   ErrorOut(hRet);
   Close;
   Exit;
  end;

 FillChar(fcaps, SizeOf(fcaps), 0);
 fcaps.dwCaps := DDSCAPS_BACKBUFFER;
 hRet := FPrimarySurface.GetAttachedSurface(fcaps, FBackGround);
 if hRet <> DD_OK then
  begin
   ErrorOut(hRet);
   Close;
   Exit;
  end;

 FTransparentImage := DDLoadBitmap(FDirectDraw, 'bart.bmp', 0, 0);
 if FTransparentImage = nil then
  begin
   ErrorOut(hRet);
   Close;
   Exit;
  end;

 DDSetColorKey (FTransparentImage, RGB(255, 0, 255));
 if hRet <> DD_OK then
  begin
   ErrorOut(hRet);
  end;
 ShowCursor(false);

 InitVertol;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 if Assigned(FVertolImage) then FVertolImage := nil;
 if Assigned(FTransparentImage) then FTransparentImage := nil;
 if Assigned(FImageSurface) then FImageSurface := nil;
 if Assigned(FBackGround) then FBackGround := nil;
 if Assigned(FPrimarySurface) then FPrimarySurface := nil;
 if Assigned(FDirectDraw) then FDirectDraw := nil;
 ShowCursor(true);
end;

procedure TForm1.FormPaint(Sender: TObject);
var
 hRet : HRESULT;
 bltfx : TDDBLTFX;
 x1,x2:Integer;
 dstR, srcR:TRect;
begin
 ZeroMemory(@bltfx, SizeOf(bltfx));
 bltfx.dwSize := sizeof(bltfx);
 bltfx.dwFillColor := 0;

 hRet := FBackGround.Blt(nil, nil, nil, DDBLT_COLORFILL or DDBLT_WAIT, @bltfx);
 if hRet = DDERR_SURFACELOST then
  begin
   FPrimarySurface._Restore;
   FImageSurface := DDLoadBitmap(FDirectDraw, '1.bmp', 0, 0);
   FTransparentImage := DDLoadBitmap(FDirectDraw, 'bart.bmp', 0, 0);
   DDSetColorKey (FTransparentImage, RGB(255, 0, 255));
   FVertolImage := DDLoadBitmap(FDirectDraw, PChar('vertol.bmp'), 0, 0);
   DDSetColorKey (FVertolImage, RGB(255, 0, 255));
  end;

 FBackGround.BltFast (175, 75, FImageSurface, nil, DDBLTFAST_WAIT);

 if Vertol1.vWaitTime<1 then
  begin
   Vertol1.vLeft:=Vertol1.vLeft+2;
   inc(Vertol1.vState);
   if Vertol1.vState>2 then Vertol1.vState:=0;

   if Vertol1.vLeft<0 then X1:=Abs(Vertol1.vLeft)
   else X1:=0;
   if Vertol1.vLeft+200>800 then X2:=800-Vertol1.vLeft
   else X2:=200;
   srcR:=Rect(X1, Vertol1.vState*80, X2, Vertol1.vState*80+80);

   if Vertol1.vLeft<0 then X1:=0
   else X1:=Vertol1.vLeft;
   if Vertol1.vLeft+200>799 then X2:=799
   else X2:=Vertol1.vLeft+200;
   dstR:=Rect(X1 ,Vertol1.vTop,X2,Vertol1.vTop+80);

   FBackGround.Blt(@dstR, FVertolImage, @srcR, DDBLT_KEYSRC, nil);

   if Vertol1.vLeft>800 then
    begin
     Vertol1.vWaitTime:=200;
     Vertol1.vLeft:=-200;
    end;
  end
 else
  Vertol1.vWaitTime:=Vertol1.vWaitTime-1;

 FBackGround.BltFast(mouseX-20, mouseY-47, FTransparentImage, nil,
                  DDBLTFAST_WAIT or DDBLTFAST_SRCCOLORKEY);
 FPrimarySurface.Flip(nil, DDFLIP_WAIT);
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if Key=VK_ESCAPE then Close;
end;

procedure TForm1.ApplicationEvents1Restore(Sender: TObject);
begin
 WindowState := wsMaximized;
 FActive := True;
end;

procedure TForm1.ApplicationEvents1Deactivate(Sender: TObject);
begin
 Application.Minimize;
 FActive := false;
end;

procedure TForm1.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
 mouseX := X;
 mouseY := Y;
 FormPaint(nil);
end;

procedure TForm1.InitVertol;
begin
 FVertolImage := DDLoadBitmap(FDirectDraw, PChar('vertol.bmp'), 0, 0);
 DDSetColorKey (FVertolImage, RGB(255, 0, 255));

 Vertol1.vLeft:=-200;
 Vertol1.vTop:=30;
 Vertol1.vState:=0;
 Vertol1.vWaitTime:=100;
end;

procedure TForm1.ApplicationEvents1Idle(Sender: TObject;
  var Done: Boolean);
begin
 if FActive <> True then exit;
 FormPaint(nil);
 Done := False;
end;

procedure TForm1.ApplicationEvents1Activate(Sender: TObject);
begin
 FActive:=true;
end;

end.
